package kim.apidoc.core.parser;

import kim.apidoc.core.common.helper.OptionalHelper;
import kim.apidoc.core.schema.Chapter;
import kim.apidoc.core.schema.Node;
import kim.apidoc.core.schema.Project;
import kim.apidoc.core.schema.Section;
import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
import com.github.javaparser.ast.body.MethodDeclaration;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;

public class VisitorParser extends VoidVisitorAdapter<Node> {

    private ParserStrategy parserStrategy;

    public void setParserStrategy(ParserStrategy parserStrategy) {
        this.parserStrategy = parserStrategy;
    }

    @Override
    public void visit(final ClassOrInterfaceDeclaration n, final Node arg) {

        if (arg instanceof Project && parserStrategy.accept(n)) {
            Project project = (Project) arg;
            Chapter chapter = new Chapter();
            n.getFullyQualifiedName().ifPresent(chapter::setId);
            chapter.setName(n.getNameAsString());
            n.getComment().ifPresent(chapter::accept);

            OptionalHelper.any(chapter.getTag("book"),chapter.getTag("group"))
                    .ifPresent(tag -> chapter.setBookName(tag.getContent()));

            parserStrategy.visit(n, chapter);
            project.addChapter(chapter);
            super.visit(n, chapter);
        }
    }

    @Override
    public void visit(final MethodDeclaration n, final Node arg) {
        if (arg instanceof Chapter && parserStrategy.accept(n)) {
            Chapter chapter = (Chapter) arg;
            Section section = new Section();
            section.setId(n.getNameAsString());
            section.setName(n.getNameAsString());
            section.setIndex(chapter.getSections().size());
            n.getComment().ifPresent(section::accept);

            parserStrategy.visit(n, chapter, section);
            chapter.getSections().add(section);
            super.visit(n, section);
        }
    }

}
